Package: FELLA
Type: Package
Title: Interpretation and enrichment for metabolomics data
Version: 1.32.0
Date: 2019-10-19
Authors@R: c(
    person(
        "Sergio", "Picart-Armada", role = c("aut", "cre"),
        email = "sergi.picart@upc.edu"), 
    person(
        "Francesc", "Fernandez-Albert", role = c("aut"),
        email = "francesc.fernandez.albert@upc.edu"),
    person(
        "Alexandre", "Perera-Lluna", role = c("aut"),
        email = "alexandre.perera@upc.edu"))
Maintainer: Sergio Picart-Armada <sergi.picart@upc.edu>
Description: Enrichment of metabolomics data using KEGG entries. 
    Given a set of affected compounds, FELLA suggests affected
    reactions, enzymes, modules and pathways using label propagation 
    in a knowledge model network. The resulting subnetwork 
    can be visualised and exported. 
License: GPL-3
LazyLoad: yes
Imports: methods, igraph, Matrix, KEGGREST, plyr, stats, graphics,
        utils
Enhances:
Depends: R (>= 3.5.0)
Suggests: shiny, DT, magrittr, visNetwork, knitr, BiocStyle, rmarkdown,
        testthat, biomaRt, org.Hs.eg.db, org.Mm.eg.db, AnnotationDbi,
        GOSemSim
VignetteBuilder: knitr
RoxygenNote: 6.1.1
Encoding: UTF-8
biocViews: Software, Metabolomics, GraphAndNetwork, KEGG, GO, Pathways,
        Network, NetworkEnrichment
Collate: 'AllArguments.R' 'AllClasses.R' 'AllMethods.R'
        'generateResultsTable.R' 'generateEnzymesTable.R'
        'generateResultsGraph.R' 'exportResults.R' 'addGOToGraph.R'
        'buildGraphFromKEGGREST.R' 'buildDataFromGraph.R'
        'defineCompounds.R' 'doc-data.R' 'doc-package.R'
        'runHypergeom.R' 'runDiffusion.R' 'runPagerank.R' 'enrich.R'
        'get-.R' 'is-.R' 'launchApp.R' 'list-.R' 'loadKEGGdata.R'
        'plotBipartite.R' 'plotGraph.R' 'plotLegend.R'
git_url: https://git.bioconductor.org/packages/FELLA
git_branch: RELEASE_3_23
git_last_commit: ed22719
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-28 22:34:20 UTC; biocbuild
Author: Sergio Picart-Armada [aut, cre],
  Francesc Fernandez-Albert [aut],
  Alexandre Perera-Lluna [aut]
