% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/list-.R
\name{listApprox}
\alias{listApprox}
\title{List of approximations}
\usage{
listApprox()
}
\value{
Character vector
}
\description{
Available approximations for the analysis
}
\examples{
listApprox()

}
