% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/list-.R
\name{listMethods}
\alias{listMethods}
\title{List of methods}
\usage{
listMethods()
}
\value{
Character vector
}
\description{
Methods available for the analysis
}
\examples{
listMethods()

}
